/***
 * User space tests
 */

#include <librt.h>
#include "main.h"

#include "condvar1/condvar1.h"
#include "condvar2/condvar2.h"
#include "condvar3/condvar3.h"
#include "io1/io1.h"
#include "malloc1/malloc1.h"
#include "mutex1/mutex1.h"
#include "mutex2/mutex2.h"
#include "mutex3/mutex3.h"
#include "mutex4/mutex4.h"
#include "rwlock1/rwlock1.h"
#include "rwlock2/rwlock2.h"
#include "rwlock3/rwlock3.h"
#include "rwlock4/rwlock4.h"
#include "semaphore1/semaphore1.h"
#include "semaphore2/semaphore2.h"
#include "semaphore3/semaphore3.h"
#include "semaphore4/semaphore4.h"
#include "thread1/thread1.h"
#include "thread2/thread2.h"
#include "uspace1/uspace1.h"
#include "uspace2/uspace2.h"
#include "uspace3/uspace3.h"

/* Number of tests available */
#define TESTS 22

/* Test functions and descriptions */
static test_t tests[TESTS] = {
	{
		test_uspace1,
		"User space protection check",
		0
	},
	{
		test_uspace2,
		"User space memory protection check",
		0
	},
	{
		test_uspace3,
		"User space stack overflow test",
		0
	},
	{
		test_thread1,
		"Thread test #1",
		1
	},
	{
		test_thread2,
		"Thread test #2",
		1
	},
	{
		test_condvar1,
		"Condition variable test #1",
		1
	},
	{
		test_condvar2,
		"Condition variable test #2",
		1
	},
	{
		test_condvar3,
		"Condition variable test #3",
		1
	},
	{
		test_io1,
		"Input/Output test #1",
		1
	},
	{
		test_malloc1,
		"Malloc test #1",
		1
	},
	{
		test_mutex1,
		"Mutex test #1",
		1
	},
	{
		test_mutex2,
		"Mutex test #2",
		1
	},
	{
		test_mutex3,
		"Mutex test #3",
		1
	},
	{
		test_mutex4,
		"Mutex test #4",
		1
	},
	{
		test_rwlock1,
		"Read/Write lock test #1",
		1
	},
	{
		test_rwlock2,
		"Read/Write lock test #2",
		1
	},
	{
		test_rwlock3,
		"Read/Write lock test #3",
		1
	},
	{
		test_rwlock4,
		"Read/Write lock test #4",
		1
	},
	{
		test_semaphore1,
		"Semaphore test #1",
		1
	},
	{
		test_semaphore2,
		"Semaphore test #2",
		1
	},
	{
		test_semaphore3,
		"Semaphore test #3",
		1
	},
	{
		test_semaphore4,
		"Semaphore test #4",
		1
	}
};


/* Convert test intex into key */
char get_key (unsigned int index)
{
	if (index < 10)
		return ('0' + index);
	
	return ('a' + index);
}

/* Convert key into test index */
unsigned int get_index(char key)
{
	if ((key >= '0') && (key <= '9'))
		return ('0' - key);
	
	return ('a' - key);
}

/* Print listing of available tests */
void list_tests (void)
{
	unsigned int index;
	
	for (index = 0; index < TESTS; index++)
		printf("%c: %s%s\n", get_key(index), tests[index].desc, (tests[index].safe ? "" : " (unsafe)"));
	
	printf("%c: run all safe tests sequentially\n", get_key(index));
}

/* Run a single test and print result */
char * run_test (test_t * test)
{
	printf("%s:\n", test->desc);
	char * ret = test->entry();
	
	if (ret == NULL)
		printf("Test passed\n");
	else
		printf("%s\n", ret);
	
	return ret;
}


int main (void)
{
	printf("\nUser space test suite\n");
	
	char key = 0;
	while (key != 'q') {
		list_tests();
		printf("> ");
		key = getc();
		printf("%c\n", key);
		
		if (key != 'q') {
			unsigned int index = get_index(key);
			
			if (index < TESTS) /* Run a single test */
				run_test(&tests[index]);
			else if (index == TESTS) {
				
				/* Run all safe tests, break on failure */
				for (index = 0; index < TESTS; index++) {
					if (tests[index].safe) {
						char * ret = run_test(&tests[index]);
						if (ret == NULL)
							printf("\n");
						else
							break;
					}
				}
				
			} else
				printf("Unknown test\n");
		}
	}
	
	return 0;
}
